#include "Arduino.h"

typedef enum direction
{
    CLOCKWISE = 1,
    COUNTERCLOCKWISE = 0
} STEPPER_DIRECTION;

class StepperDM556T
{
public:
    //
    void init(int direction_pin, int PULSE_PIN, int enable_pin);
    void step(bool dir, int steps, int speed);
    void start_movement(STEPPER_DIRECTION dir, int steps, int speed);
    void update_movement();
    bool is_movement_complete();
    void enable();
    void disable();
    void update();
    void set_steps_remaining(int steps);
    void set_direction(STEPPER_DIRECTION dir);
    bool is_enabled();
private:
    int DIRECTION_PIN;
    int PULSE_PIN;
    int ENABLE_PIN;
    int stepsRemaining;
    int stepSpeed;
    unsigned long startTime;
    bool pulseHigh;
    STEPPER_DIRECTION direction;
    bool enabled;
};

void StepperDM556T::init(int direction_pin, int PULSE_PIN, int enable_pin)
{
    // Specify the pins for the stepper motor
    this->DIRECTION_PIN = direction_pin;
    this->ENABLE_PIN = enable_pin;
    this->PULSE_PIN = PULSE_PIN;
    this->stepsRemaining = 0;
    this->stepSpeed = 0;
    this->direction = CLOCKWISE;
    this->startTime = 0;
    this->pulseHigh = false;
    // Set them up to be outputs
    pinMode(this->DIRECTION_PIN, OUTPUT);
    pinMode(this->ENABLE_PIN, OUTPUT);
    pinMode(this->PULSE_PIN, OUTPUT);
    digitalWrite(this->DIRECTION_PIN, LOW);
    digitalWrite(this->PULSE_PIN, LOW);
    digitalWrite(this->ENABLE_PIN, LOW);
    this->enabled = false;
    this->disable();
}
bool StepperDM556T::is_enabled(){
  return this->enabled;
}

void StepperDM556T::enable()
{
    this->enabled = true;
    // Enable the driver
    digitalWrite(this->DIRECTION_PIN, HIGH);
    digitalWrite(this->PULSE_PIN, HIGH);
    delay(1);
    digitalWrite(this->ENABLE_PIN, LOW);
    delay(1); // delay for 1 ms to let the driver wakeup.
    digitalWrite(this->DIRECTION_PIN, LOW);
    delay(1); // delay for 1 ms to let the driver wakeup.
    digitalWrite(this->DIRECTION_PIN, HIGH);
    delay(250);
}
void StepperDM556T::disable()
{
    this->enabled = false;
    delay(10);
    // disable the driver, meaning shut off.
    digitalWrite(this->DIRECTION_PIN, HIGH);
    digitalWrite(this->PULSE_PIN, HIGH);
    digitalWrite(this->ENABLE_PIN, HIGH);
}

void StepperDM556T::start_movement(STEPPER_DIRECTION dir, int steps, int speed)
{
    // This only updates the details of the movmenet, the update_movement method does the actual moving.
    this->stepsRemaining = steps;
    this->direction = dir;

    // Speed is a delay value for how long it waits before sending another pulse
    this->stepSpeed = speed;
    this->startTime = millis();
    this->pulseHigh = false;
    // Set the direction pin
    digitalWrite(this->DIRECTION_PIN, dir);
    delayMicroseconds(5);
    digitalWrite(this->PULSE_PIN, LOW);
}

void StepperDM556T::set_steps_remaining(int steps)
{
  this->stepsRemaining = steps;
}

void StepperDM556T::set_direction(STEPPER_DIRECTION dir){
  digitalWrite(this->DIRECTION_PIN, dir);
}

unsigned long myTime = 0;
void StepperDM556T::update_movement()
{
    if (this->stepsRemaining > 0)
    {
        if (this->pulseHigh == false)
        {
            // delay for 1 millisecond for the pulse to register
            if ((millis() - this->startTime) > 1)
            {
                digitalWrite(this->PULSE_PIN, true);
                this->pulseHigh = true;
                this->startTime = millis();
                this->stepsRemaining -= 1;
            }
        }
        else
        {
            // Now wait until the speed time has elapsed
            if ((millis() - this->startTime) > this->stepSpeed)
            {
                this->pulseHigh = false;
                this->startTime = millis();
                digitalWrite(this->PULSE_PIN, false);
            }
        }
    }
    else
    {
      this->stepsRemaining = 0;
    }
}

bool StepperDM556T::is_movement_complete()
{
    return this->stepsRemaining == 0;
}



//This is used to debounce the values from the controller to check if the value has gone over a threshold once or not.
class BoolDebounce
{
  public:
    void ValueDebounce(bool value);
    void ValueDebounce();
    bool has_value_rose();
    void update_value(bool new_value);
    bool is_value_high();
  private:
   int stored_value;
   bool rose;

};

void BoolDebounce::ValueDebounce(bool value){
  this->stored_value = value;
}

void BoolDebounce::ValueDebounce(){
  this->stored_value = 0;
}

bool BoolDebounce::has_value_rose(){
  bool to_ret = this->rose;
  this->rose = false;
  return to_ret;
}

bool BoolDebounce::is_value_high(){
  return this->stored_value;
}
void BoolDebounce::update_value(bool new_value){
  //If the new value is true and the old value is false then it has fallen.
  if (new_value and !this->stored_value){
    this->rose = true;
  }
//  //If the new value is false and the old value is true it fell.
//  else if (!new_value and this->stored_value)
//  {
//    this->rose = false
//  }

  this->stored_value = new_value;
}
